@@ -3,7 +3,6 @@
 use core::convert::From;
 use core::fmt::{self, Debug, Display};
 use core::result;
-use std::error;
 
 /// Result type with crate [Error].
 pub type Result<T> = result::Result<T, Error>;
@@ -23,6 +22,7 @@ impl Error {
     }
 
     /// If the error is due to the JWT being malformed.
+    #[must_use]
     pub fn is_malformed_jwt(&self) -> bool {
         matches!(self.err.code, ErrorCode::MalformedJwt)
     }
@@ -39,39 +39,41 @@ impl Error {
         }
     }
     /// If the error is due to an invalid signature.
+    #[must_use]
     pub fn is_invalid_signature(&self) -> bool {
         matches!(self.err.code, ErrorCode::InvalidSignature)
     }
 
     /// If the error is due to a part not being correctly base64 encoded.
+    #[must_use]
     pub fn is_base64_decode_error(&self) -> bool {
         matches!(self.err.code, ErrorCode::Base64Decode(_))
     }
 
     /// If the error is due to an invalid key.
+    #[must_use]
     pub fn is_key_rejected(&self) -> bool {
         matches!(self.err.code, ErrorCode::KeyRejected)
     }
-}
 
-impl error::Error for Error {
-    fn description(&self) -> &str {
-        match self.err.code {
-            ErrorCode::Base64Decode(_) => "base64 decode error",
-            ErrorCode::InvalidSignature => "invalid signature",
-            ErrorCode::KeyRejected => "key rejected",
-            ErrorCode::MalformedJwt => "malformed jwt",
-            ErrorCode::Unspecified => "unspecified error",
+    #[must_use]
+    pub(crate) fn unspecified() -> Self {
+        Error {
+            err: Box::new(ErrorImpl {
+                code: ErrorCode::Unspecified,
+            }),
         }
     }
+}
 
-    fn source(&self) -> Option<&(dyn error::Error + 'static)> {
+impl std::error::Error for Error {
+    fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
         match self.err.code {
             ErrorCode::Base64Decode(ref err) => Some(err),
-            ErrorCode::InvalidSignature => None,
-            ErrorCode::KeyRejected => None,
-            ErrorCode::MalformedJwt => None,
-            ErrorCode::Unspecified => None,
+            ErrorCode::InvalidSignature
+            | ErrorCode::KeyRejected
+            | ErrorCode::MalformedJwt
+            | ErrorCode::Unspecified => None,
         }
     }
 }
@@ -147,10 +149,11 @@ pub(crate) enum ErrorCode {
 
 impl Display for ErrorCode {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        match *self {
+        match self {
             ErrorCode::Base64Decode(ref error) => Display::fmt(error, f),
-            ErrorCode::InvalidSignature => f.write_str("invalid signature"),
-            ErrorCode::KeyRejected => f.write_str("invalid signature"),
+            ErrorCode::InvalidSignature | ErrorCode::KeyRejected => {
+                f.write_str("invalid signature")
+            }
             ErrorCode::MalformedJwt => f.write_str("malformed jwt"),
             ErrorCode::Unspecified => f.write_str("unspecified error"),
         }
