@@ -1,7 +1,7 @@
 //! Sign JWTs using various signature algorithms.
 //!
-//! The [encode_and_sign](super::encode_and_sign) and
-//! [serialize_encode_and_sign](super::serialize_encode_and_sign) functions sign
+//! The [`encode_and_sign`](crate::encode_and_sign) and
+//! [`serialize_encode_and_sign`](crate::serialize_encode_and_sign) functions sign
 //! JWTs.
 //!
 //! Both functions require a [Signer] which abstracts the implementation and
@@ -14,22 +14,16 @@
 //!
 //! See the implementation modules for specific examples.
 
-use crate::error::Result;
-
 /// A signature which can be represented by bytes.
-pub trait Signature: AsRef<[u8]> + private::Private {}
+pub trait Signature: AsRef<[u8]> {}
 
-impl private::Private for Vec<u8> {}
 impl Signature for Vec<u8> {}
 
-// impl private::Private for &[u8] {}
-// impl Signature for &[u8] {}
+impl Signature for &[u8] {}
 
-// impl private::Private for String {}
-// impl Signature for String {}
+impl Signature for String {}
 
-// impl private::Private for &str {}
-// impl Signature for &str {}
+impl Signature for &str {}
 
 /// A type which can sign a byte buffer.
 ///
@@ -39,12 +33,19 @@ impl Signature for Vec<u8> {}
 /// In other cases, a new type composed of multiple fields may be needed because
 /// the signing key's sign method may require more parameters (e.g. a random
 /// number generator).
-pub trait Signer: private::Private {
+pub trait Signer {
     /// Returned signature type which implmenets the [Signature] trait.
     type Signature: Signature;
 
+    /// Returns an error.
+    type Error;
+
     /// Returns a signature from a byte buffer.
-    fn sign(&self, bytes: &[u8]) -> Result<Self::Signature>;
+    ///
+    /// # Errors
+    ///
+    /// Returns an error dependent on the signer.
+    fn sign(&self, bytes: &[u8]) -> Result<Self::Signature, Self::Error>;
 }
 
 impl<T> Signer for &T
@@ -53,17 +54,13 @@ where
 {
     type Signature = T::Signature;
 
-    fn sign(&self, bytes: &[u8]) -> Result<Self::Signature> {
+    type Error = T::Error;
+
+    fn sign(&self, bytes: &[u8]) -> Result<Self::Signature, Self::Error> {
         T::sign(self, bytes)
     }
 }
 
-mod private {
-    pub trait Private {}
-
-    impl<T> Private for &T where T: Private {}
-}
-
 #[cfg(feature = "p256")]
 #[cfg_attr(docsrs, doc(cfg(feature = "p256")))]
 pub mod p256;
