@@ -4,9 +4,9 @@
 //!
 //! | Algorithm | Type | Wrapper Type |
 //! | --------- | ---- | ------------ |
-//! | es256     | [::ring::signature::EcdsaKeyPair] | [EcdsaKeyPairSigner] |
-//! | hs256     | [::ring::hmac::Key]               | [HmacKeySigner] |
-//! | rs256     | [::ring::signature::RsaKeyPair]   | [RsaKeyPairSigner] |
+//! | es256     | [`::ring::signature::EcdsaKeyPair`] | [`EcdsaKeyPairSigner`] |
+//! | hs256     | [`::ring::hmac::Key`]               | [`HmacKeySigner`] |
+//! | rs256     | [`::ring::signature::RsaKeyPair`]   |`RsaKeyPairSigner`er] |
 //!
 //! # Examples
 //!
@@ -86,7 +86,7 @@
 //! # let header = "{\"alg\":\"RS256\",\"typ\":\"JWT\"}";
 //! # let claims = "{\"sub\":\"1234567890\",\"name\":\"Jane Doe\",\"iat\":1516239022}";
 //! # fn convert_pkcs8_pem_to_der(private_key: &str) -> impl AsRef<[u8]> {
-//! #   use rsa::pkcs8::{ToPrivateKey, FromPrivateKey};
+//! #   use rsa::pkcs8::{DecodePrivateKey, EncodePrivateKey};
 //! #   let private_key = ::rsa::RsaPrivateKey::from_pkcs8_pem(&private_key).unwrap();
 //! #   private_key.to_pkcs8_der().unwrap()
 //! # }
@@ -143,28 +143,32 @@
 //! ```
 
 use super::Signature;
-use crate::{
-    algorithm::{Algorithm, Es256, Hs256, Rs256},
-    error::{Error, Result},
-};
+use crate::algorithm::{Algorithm, Es256, Hs256, Rs256};
 use core::marker::PhantomData;
 use ring::rand::SecureRandom;
 use ring::signature::EcdsaKeyPair;
 
 impl Signature for ::ring::signature::Signature {}
-impl super::private::Private for ::ring::signature::Signature {}
 impl Signature for ::ring::hmac::Tag {}
-impl super::private::Private for ::ring::hmac::Tag {}
 
-mod private {
-    pub trait Private {}
-    impl<T> Private for &T where T: Private {}
-}
-
-pub trait EcdsaKey: private::Private {
+/// An `ECDSA` key.
+pub trait EcdsaKey {
+    /// Signature type returned.
     type Signature: Signature;
 
-    fn sign<B>(&self, secure_random: &dyn SecureRandom, bytes: B) -> Result<Self::Signature>
+    /// Error type returned.
+    type Error;
+
+    /// Signs the bytes with the key.
+    ///
+    /// # Errors
+    ///
+    /// Returns an underlying implementation error.
+    fn sign<B>(
+        &self,
+        secure_random: &dyn SecureRandom,
+        bytes: B,
+    ) -> Result<Self::Signature, Self::Error>
     where
         B: AsRef<[u8]>;
 }
@@ -175,7 +179,13 @@ where
 {
     type Signature = T::Signature;
 
-    fn sign<B>(&self, secure_random: &dyn SecureRandom, bytes: B) -> Result<Self::Signature>
+    type Error = T::Error;
+
+    fn sign<B>(
+        &self,
+        secure_random: &dyn SecureRandom,
+        bytes: B,
+    ) -> Result<Self::Signature, Self::Error>
     where
         B: AsRef<[u8]>,
     {
@@ -183,20 +193,24 @@ where
     }
 }
 
-impl private::Private for ::ring::signature::EcdsaKeyPair {}
-
 impl EcdsaKey for ::ring::signature::EcdsaKeyPair {
     type Signature = ::ring::signature::Signature;
 
-    fn sign<B>(&self, secure_random: &dyn SecureRandom, bytes: B) -> Result<Self::Signature>
+    type Error = ::ring::error::Unspecified;
+
+    fn sign<B>(
+        &self,
+        secure_random: &dyn SecureRandom,
+        bytes: B,
+    ) -> Result<Self::Signature, Self::Error>
     where
         B: AsRef<[u8]>,
     {
-        EcdsaKeyPair::sign(self, secure_random, bytes.as_ref()).map_err(|_| todo!())
+        EcdsaKeyPair::sign(self, secure_random, bytes.as_ref())
     }
 }
 
-/// Wrapper for [::ring::signature::EcdsaKeyPair].
+/// Wrapper for [`::ring::signature::EcdsaKeyPair`].
 ///
 /// # Examples
 ///
@@ -263,20 +277,13 @@ where
     algorithm: PhantomData<A>,
 }
 
-impl<K, R, A> super::private::Private for EcdsaKeyPairSigner<K, R, A>
-where
-    K: EcdsaKey,
-    R: SecureRandom,
-    A: Algorithm,
-{
-}
-
 impl<K, R, A> EcdsaKeyPairSigner<K, R, A>
 where
     K: EcdsaKey,
     R: SecureRandom,
     A: Algorithm,
 {
+    /// Returns the inner key and `SecureRandom`.
     pub fn into_inner(self) -> (K, R) {
         (self.key_pair, self.secure_random)
     }
@@ -287,6 +294,7 @@ where
     K: EcdsaKey,
     R: SecureRandom,
 {
+    /// Creates a new `Es256` key signer.
     pub fn with_es256(key_pair: K, secure_random: R) -> EcdsaKeyPairSigner<K, R, Es256> {
         Self {
             key_pair,
@@ -304,15 +312,27 @@ where
 {
     type Signature = K::Signature;
 
-    fn sign(&self, bytes: &[u8]) -> Result<Self::Signature> {
+    type Error = K::Error;
+
+    fn sign(&self, bytes: &[u8]) -> Result<Self::Signature, Self::Error> {
         self.key_pair.sign(&self.secure_random, bytes)
     }
 }
 
-pub trait HmacKey: private::Private {
+/// An `Hmac` key.
+pub trait HmacKey {
+    /// Signature type returned.
     type Signature: Signature;
 
-    fn sign<B>(&self, bytes: B) -> Result<Self::Signature>
+    /// Error type returned.
+    type Error;
+
+    /// Signs the bytes with the key.
+    ///
+    /// # Errors
+    ///
+    /// Returns an underlying implementation error.
+    fn sign<B>(&self, bytes: B) -> Result<Self::Signature, Self::Error>
     where
         B: AsRef<[u8]>;
 }
@@ -323,7 +343,9 @@ where
 {
     type Signature = T::Signature;
 
-    fn sign<B>(&self, bytes: B) -> Result<Self::Signature>
+    type Error = T::Error;
+
+    fn sign<B>(&self, bytes: B) -> Result<Self::Signature, Self::Error>
     where
         B: AsRef<[u8]>,
     {
@@ -334,16 +356,17 @@ where
 impl HmacKey for ::ring::hmac::Key {
     type Signature = ::ring::hmac::Tag;
 
-    fn sign<B>(&self, bytes: B) -> Result<Self::Signature>
+    type Error = ();
+
+    fn sign<B>(&self, bytes: B) -> Result<Self::Signature, Self::Error>
     where
         B: AsRef<[u8]>,
     {
         Ok(::ring::hmac::sign(self, bytes.as_ref()))
     }
 }
-impl private::Private for ::ring::hmac::Key {}
 
-/// Wrapper for [::ring::hmac::Key].
+/// Wrapper for [`::ring::hmac::Key`].
 ///
 /// # Examples
 ///
@@ -360,6 +383,7 @@ impl private::Private for ::ring::hmac::Key {}
 /// let jwt = min_jwt::encode_and_sign(header, claims, &signer);
 /// # Ok::<(), min_jwt::Error>(())
 /// ```
+#[derive(Debug)]
 pub struct HmacKeySigner<K, A>
 where
     K: HmacKey,
@@ -369,18 +393,12 @@ where
     algorithm: PhantomData<A>,
 }
 
-impl<K, A> super::private::Private for HmacKeySigner<K, A>
-where
-    K: HmacKey,
-    A: Algorithm,
-{
-}
-
 impl<K, A> HmacKeySigner<K, A>
 where
     K: HmacKey,
     A: Algorithm,
 {
+    /// Returns the inner key.
     pub fn into_inner(self) -> K {
         self.key
     }
@@ -393,7 +411,9 @@ where
 {
     type Signature = K::Signature;
 
-    fn sign(&self, bytes: &[u8]) -> Result<Self::Signature> {
+    type Error = K::Error;
+
+    fn sign(&self, bytes: &[u8]) -> Result<Self::Signature, Self::Error> {
         self.key.sign(bytes)
     }
 }
@@ -402,6 +422,7 @@ impl<K> HmacKeySigner<K, Hs256>
 where
     K: HmacKey,
 {
+    /// Creates a new `Hs256` key signer.
     pub fn with_hs256(key: K) -> HmacKeySigner<K, Hs256> {
         Self {
             key,
@@ -410,10 +431,24 @@ where
     }
 }
 
-pub trait RsaKey: private::Private {
+/// An `RSA` key.
+pub trait RsaKey {
+    /// Signature type returned.
     type Signature: Signature;
 
-    fn sign<B>(&self, secure_random: &dyn SecureRandom, bytes: B) -> Result<Self::Signature>
+    /// Error type returned.
+    type Error;
+
+    /// Signs the bytes with the key.
+    ///
+    /// # Errors
+    ///
+    /// Returns an underlying implementation error.
+    fn sign<B>(
+        &self,
+        secure_random: &dyn SecureRandom,
+        bytes: B,
+    ) -> Result<Self::Signature, Self::Error>
     where
         B: AsRef<[u8]>;
 }
@@ -424,7 +459,13 @@ where
 {
     type Signature = T::Signature;
 
-    fn sign<B>(&self, secure_random: &dyn SecureRandom, bytes: B) -> Result<Self::Signature>
+    type Error = T::Error;
+
+    fn sign<B>(
+        &self,
+        secure_random: &dyn SecureRandom,
+        bytes: B,
+    ) -> Result<Self::Signature, Self::Error>
     where
         B: AsRef<[u8]>,
     {
@@ -435,7 +476,13 @@ where
 impl RsaKey for ::ring::signature::RsaKeyPair {
     type Signature = Vec<u8>;
 
-    fn sign<B>(&self, secure_random: &dyn SecureRandom, bytes: B) -> Result<Self::Signature>
+    type Error = ::ring::error::Unspecified;
+
+    fn sign<B>(
+        &self,
+        secure_random: &dyn SecureRandom,
+        bytes: B,
+    ) -> Result<Self::Signature, Self::Error>
     where
         B: AsRef<[u8]>,
     {
@@ -446,14 +493,12 @@ impl RsaKey for ::ring::signature::RsaKeyPair {
             secure_random,
             bytes.as_ref(),
             &mut signature,
-        )
-        .map_err(|_| Error::invalid_signature())?;
+        )?;
         Ok(signature)
     }
 }
-impl private::Private for ::ring::signature::RsaKeyPair {}
 
-/// Wrapper for [::ring::signature::RsaKeyPair].
+/// Wrapper for [`::ring::signature::RsaKeyPair`].
 ///
 /// # Examples
 ///
@@ -465,7 +510,7 @@ impl private::Private for ::ring::signature::RsaKeyPair {}
 /// # let header = "{\"alg\":\"RS256\",\"typ\":\"JWT\"}";
 /// # let claims = "{\"sub\":\"1234567890\",\"name\":\"Jane Doe\",\"iat\":1516239022}";
 /// # fn convert_pkcs8_pem_to_der(private_key: &str) -> impl AsRef<[u8]> {
-/// #   use rsa::pkcs8::{ToPrivateKey, FromPrivateKey};
+/// #   use rsa::pkcs8::{DecodePrivateKey, EncodePrivateKey};
 /// #   let private_key = ::rsa::RsaPrivateKey::from_pkcs8_pem(&private_key).unwrap();
 /// #   private_key.to_pkcs8_der().unwrap()
 /// # }
@@ -532,20 +577,13 @@ where
     alg: PhantomData<A>,
 }
 
-impl<K, R, A> super::private::Private for RsaKeyPairSigner<K, R, A>
-where
-    K: RsaKey,
-    R: SecureRandom,
-    A: Algorithm,
-{
-}
-
 impl<K, R, A> RsaKeyPairSigner<K, R, A>
 where
     K: RsaKey,
     R: SecureRandom,
     A: Algorithm,
 {
+    /// Returns the inner key and `SecureRandom`.
     pub fn into_inner(self) -> (K, R) {
         (self.key_pair, self.secure_random)
     }
@@ -559,7 +597,9 @@ where
 {
     type Signature = K::Signature;
 
-    fn sign(&self, bytes: &[u8]) -> Result<Self::Signature> {
+    type Error = K::Error;
+
+    fn sign(&self, bytes: &[u8]) -> Result<Self::Signature, Self::Error> {
         self.key_pair.sign(&self.secure_random, bytes)
     }
 }
