@@ -4,7 +4,7 @@
 //!
 //! | Algorithm | Type | Wrapper Type |
 //! | --------- | ---- | ------------ |
-//! | rs256     | [::rsa::RsaPrivateKey] | [RsaPrivateKeySigner] |
+//! | rs256     | [`::rsa::RsaPrivateKey`] |`PrivateKeySigner`er] |
 //!
 //! Note that you will also need the `sha2` feature enabled.
 //!
@@ -14,14 +14,14 @@
 //!
 //! The `pem` feature on the `rsa` dependency must be enabled.
 //!
-//! See the [::rsa::pkcs8::FromPrivateKey] trait for more methods.
+//! See the [`::rsa::pkcs8::DecodePrivateKey`] trait for more methods.
 //!
 //! ```
 //! # #[cfg(feature="sha2")]
 //! # fn try_main() -> Result<(), min_jwt::error::Error> {
 //! # let header = "{\"alg\":\"RS256\",\"typ\":\"JWT\"}";
 //! # let claims = "{\"sub\":\"1234567890\",\"name\":\"Jane Doe\",\"iat\":1516239022}";
-//! use rsa::pkcs8::FromPrivateKey;
+//! use rsa::pkcs8::DecodePrivateKey;
 //!
 //! let private_key =
 //! "-----BEGIN PRIVATE KEY-----
@@ -55,7 +55,7 @@
 //!
 //! let private_key = ::rsa::RsaPrivateKey::from_pkcs8_pem(&private_key).unwrap();
 //!
-//! let signer = min_jwt::sign::rsa::RsaPrivateKeySigner::with_rs256(&private_key);
+//! let signer = min_jwt::sign::rsa::PrivateKeySigner::with_rs256(&private_key);
 //! let jwt = min_jwt::encode_and_sign(header, claims, &signer)?;
 //! # assert_eq!("eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkphbmUgRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ.BV5tgihZQo_CCSJuwSmespFnUPVcE1tZ52td6wYfB6j-YuKanRuHD4hJZPO-fN2GYe492aU4FDFVqVqC3cZcv5sZgkZolPgAhXVlQymw___vmvcodWv7xLjZBr4INpzb4FPUkaNhAd1LvF28CXHx0aNvoyyOo4i_AR1ZYBk6CbsCrVj7XxdsVmP3VBpXLSFKcit0FrWBs_sP0-g2qQDIKZ5w9HNiv4H3fU5NZ_TNKRKIQkwMJ1hvI_JbacIZ9uk2oYZ6LwV_NMeh0EqIwRg1EsH6TcdXhzLRozVa1fbej9hd2-AOGxZTba3LQtBAEKbyEATd7N5mqtEsRvcTHzXJmw", jwt);
 //! # Ok::<(), min_jwt::Error>(())
@@ -67,30 +67,45 @@
 //! ```
 use core::marker::PhantomData;
 
-use crate::{algorithm::Algorithm, error::Result};
+use crate::algorithm::Algorithm;
 
 use super::Signature;
 
-pub trait Key: private::Private {
+/// An `RSA` key.
+pub trait Key {
+    /// Signature type returned.
     type Signature: Signature;
 
-    fn sign<B>(&self, padding: ::rsa::PaddingScheme, bytes: B) -> Result<Self::Signature>
+    /// Error type returned.
+    type Error;
+
+    /// Signs the bytes with the key.
+    ///
+    /// # Errors
+    ///
+    /// Returns an underlying implementation error.
+    fn sign<B>(
+        &self,
+        padding: ::rsa::PaddingScheme,
+        bytes: B,
+    ) -> Result<Self::Signature, Self::Error>
     where
         B: AsRef<[u8]>;
 }
 
-mod private {
-    pub trait Private {}
-    impl<T> Private for &T where T: Private {}
-}
-
 impl<T> Key for &T
 where
     T: Key,
 {
     type Signature = T::Signature;
 
-    fn sign<B>(&self, padding: ::rsa::PaddingScheme, bytes: B) -> Result<Self::Signature>
+    type Error = T::Error;
+
+    fn sign<B>(
+        &self,
+        padding: ::rsa::PaddingScheme,
+        bytes: B,
+    ) -> Result<Self::Signature, Self::Error>
     where
         B: AsRef<[u8]>,
     {
@@ -101,17 +116,21 @@ where
 impl Key for ::rsa::RsaPrivateKey {
     type Signature = Vec<u8>;
 
-    fn sign<B>(&self, padding: ::rsa::PaddingScheme, bytes: B) -> Result<Self::Signature>
+    type Error = ::rsa::errors::Error;
+
+    fn sign<B>(
+        &self,
+        padding: ::rsa::PaddingScheme,
+        bytes: B,
+    ) -> Result<Self::Signature, Self::Error>
     where
         B: AsRef<[u8]>,
     {
-        ::rsa::RsaPrivateKey::sign(self, padding, bytes.as_ref()).map_err(|_| todo!())
+        ::rsa::RsaPrivateKey::sign(self, padding, bytes.as_ref())
     }
 }
 
-impl private::Private for ::rsa::RsaPrivateKey {}
-
-/// Wrapper for [::rsa::RsaPrivateKey].
+/// Wrapper for [`::rsa::RsaPrivateKey`].
 ///
 /// # Examples
 ///
@@ -122,7 +141,7 @@ impl private::Private for ::rsa::RsaPrivateKey {}
 /// # fn try_main() -> Result<(), min_jwt::error::Error> {
 /// # let header = "{\"alg\":\"RS256\",\"typ\":\"JWT\"}";
 /// # let claims = "{\"sub\":\"1234567890\",\"name\":\"Jane Doe\",\"iat\":1516239022}";
-/// use rsa::pkcs8::FromPrivateKey;
+/// use rsa::pkcs8::DecodePrivateKey;
 ///
 /// let private_key =
 /// "-----BEGIN PRIVATE KEY-----
@@ -156,7 +175,7 @@ impl private::Private for ::rsa::RsaPrivateKey {}
 ///
 /// let private_key = ::rsa::RsaPrivateKey::from_pkcs8_pem(&private_key).unwrap();
 ///
-/// let signer = min_jwt::sign::rsa::RsaPrivateKeySigner::with_rs256(&private_key);
+/// let signer = min_jwt::sign::rsa::PrivateKeySigner::with_rs256(&private_key);
 /// let jwt = min_jwt::encode_and_sign(header, claims, &signer)?;
 /// # assert_eq!("eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkphbmUgRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ.BV5tgihZQo_CCSJuwSmespFnUPVcE1tZ52td6wYfB6j-YuKanRuHD4hJZPO-fN2GYe492aU4FDFVqVqC3cZcv5sZgkZolPgAhXVlQymw___vmvcodWv7xLjZBr4INpzb4FPUkaNhAd1LvF28CXHx0aNvoyyOo4i_AR1ZYBk6CbsCrVj7XxdsVmP3VBpXLSFKcit0FrWBs_sP0-g2qQDIKZ5w9HNiv4H3fU5NZ_TNKRKIQkwMJ1hvI_JbacIZ9uk2oYZ6LwV_NMeh0EqIwRg1EsH6TcdXhzLRozVa1fbej9hd2-AOGxZTba3LQtBAEKbyEATd7N5mqtEsRvcTHzXJmw", jwt);
 /// # Ok::<(), min_jwt::Error>(())
@@ -167,7 +186,7 @@ impl private::Private for ::rsa::RsaPrivateKey {}
 /// # }
 /// ```
 #[derive(Debug)]
-pub struct RsaPrivateKeySigner<K, A>
+pub struct PrivateKeySigner<K, A>
 where
     K: Key,
     A: Algorithm,
@@ -176,18 +195,12 @@ where
     alg: PhantomData<A>,
 }
 
-impl<K, A> super::private::Private for RsaPrivateKeySigner<K, A>
-where
-    K: Key,
-    A: Algorithm,
-{
-}
-
-impl<K, A> RsaPrivateKeySigner<K, A>
+impl<K, A> PrivateKeySigner<K, A>
 where
     K: Key,
     A: Algorithm,
 {
+    /// Returns the inner key.
     pub fn into_inner(self) -> K {
         self.key
     }
@@ -195,10 +208,11 @@ where
 
 #[cfg(feature = "sha2")]
 #[cfg_attr(docsrs, doc(cfg(feature = "sha2")))]
-impl<K> RsaPrivateKeySigner<K, crate::algorithm::Rs256>
+impl<K> PrivateKeySigner<K, crate::algorithm::Rs256>
 where
     K: Key,
 {
+    /// Creates a new `Rs256` key signer.
     pub fn with_rs256(key: K) -> Self {
         Self {
             key,
@@ -209,13 +223,15 @@ where
 
 #[cfg(feature = "sha2")]
 #[cfg_attr(docsrs, doc(cfg(feature = "sha2")))]
-impl<K> super::Signer for RsaPrivateKeySigner<K, crate::algorithm::Rs256>
+impl<K> super::Signer for PrivateKeySigner<K, crate::algorithm::Rs256>
 where
     K: Key,
 {
     type Signature = K::Signature;
 
-    fn sign(&self, bytes: &[u8]) -> Result<Self::Signature> {
+    type Error = K::Error;
+
+    fn sign(&self, bytes: &[u8]) -> Result<Self::Signature, Self::Error> {
         use sha2::{Digest, Sha256};
 
         self.key.sign(
