@@ -21,8 +21,12 @@ use crate::error::Result;
 ///
 /// In other cases, a new type composed of multiple fields may be needed because
 /// the verifying key's verify method may require more parameters.
-pub trait Verifier: private::Private {
-    /// Verifies the signature for the given message.
+pub trait Verifier {
+    /// Verifies the message with the given signature.
+    ///
+    /// # Errors
+    ///
+    /// Returns an error if the signature is not valid for the message.
     fn verify<M, S>(&self, message: M, signature: S) -> Result<()>
     where
         M: AsRef<[u8]>,
@@ -42,12 +46,6 @@ where
     }
 }
 
-mod private {
-    pub trait Private {}
-
-    impl<T> Private for &T where T: Private {}
-}
-
 #[cfg(feature = "p256")]
 #[cfg_attr(docsrs, doc(cfg(feature = "p256")))]
 pub mod p256;
