@@ -4,7 +4,7 @@
 //!
 //! | Algorithm | Type | Wrapper Type |
 //! | --------- | ---- | ------------ |
-//! | rs256     | [::rsa::RsaPublicKey] | [RsaPublicKeyVerifier] |
+//! | rs256     | [`::rsa::RsaPublicKey`] | [`PublicKeyVerifier`] |
 //!
 //! Note that you will also need the `sha2` feature enabled.
 //!
@@ -18,7 +18,7 @@
 //! # #[cfg(all(feature = "rsa", feature = "sha2"))]
 //! # fn try_main() -> Result<(), min_jwt::error::Error> {
 //! # let jwt = "eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkphbmUgRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ.BV5tgihZQo_CCSJuwSmespFnUPVcE1tZ52td6wYfB6j-YuKanRuHD4hJZPO-fN2GYe492aU4FDFVqVqC3cZcv5sZgkZolPgAhXVlQymw___vmvcodWv7xLjZBr4INpzb4FPUkaNhAd1LvF28CXHx0aNvoyyOo4i_AR1ZYBk6CbsCrVj7XxdsVmP3VBpXLSFKcit0FrWBs_sP0-g2qQDIKZ5w9HNiv4H3fU5NZ_TNKRKIQkwMJ1hvI_JbacIZ9uk2oYZ6LwV_NMeh0EqIwRg1EsH6TcdXhzLRozVa1fbej9hd2-AOGxZTba3LQtBAEKbyEATd7N5mqtEsRvcTHzXJmw";
-//! use ::rsa::pkcs8::FromPublicKey;
+//! use ::rsa::pkcs8::DecodePublicKey;
 //!
 //! let public_key =
 //! "-----BEGIN PUBLIC KEY-----
@@ -33,7 +33,7 @@
 //!
 //! let public_key = ::rsa::RsaPublicKey::from_public_key_pem(public_key).unwrap();
 //!
-//! let verifier = min_jwt::verify::rsa::RsaPublicKeyVerifier::with_rs256(public_key);
+//! let verifier = min_jwt::verify::rsa::PublicKeyVerifier::with_rs256(public_key);
 //! let result = min_jwt::verify(jwt, &verifier)?;
 //!
 //! let header = result.decode_header();
@@ -52,21 +52,19 @@ use crate::{
 };
 use core::marker::PhantomData;
 
-impl super::private::Private for ::rsa::RsaPublicKey {}
-
 /// Types which can verify a signature.
-pub trait PublicKey: private::Private {
+pub trait PublicKey {
+    /// Verifies the message with the given signature.
+    ///
+    /// # Errors
+    ///
+    /// Returns an error if the signature is not valid for the message.
     fn verify<M, S>(&self, message: M, signature: S, padding: ::rsa::PaddingScheme) -> Result<()>
     where
         M: AsRef<[u8]>,
         S: AsRef<[u8]>;
 }
 
-mod private {
-    pub trait Private {}
-    impl<T> Private for &T where T: Private {}
-}
-
 impl<T> PublicKey for &T
 where
     T: PublicKey,
@@ -91,11 +89,9 @@ impl PublicKey for ::rsa::RsaPublicKey {
     }
 }
 
-impl private::Private for ::rsa::RsaPublicKey {}
-
 /// A wrapper type which holds the key and algorithm.
 #[derive(Debug)]
-pub struct RsaPublicKeyVerifier<K, A>
+pub struct PublicKeyVerifier<K, A>
 where
     K: PublicKey,
     A: Algorithm,
@@ -104,18 +100,12 @@ where
     alg: PhantomData<A>,
 }
 
-impl<K, A> super::private::Private for RsaPublicKeyVerifier<K, A>
-where
-    K: PublicKey,
-    A: Algorithm,
-{
-}
-
-impl<K, A> RsaPublicKeyVerifier<K, A>
+impl<K, A> PublicKeyVerifier<K, A>
 where
     K: PublicKey,
     A: Algorithm,
 {
+    /// Returns the inner key.
     pub fn into_inner(self) -> K {
         self.key
     }
@@ -123,10 +113,11 @@ where
 
 #[cfg(feature = "sha2")]
 #[cfg_attr(docsrs, doc(cfg(feature = "sha2")))]
-impl<K> RsaPublicKeyVerifier<K, crate::algorithm::Rs256>
+impl<K> PublicKeyVerifier<K, crate::algorithm::Rs256>
 where
     K: PublicKey,
 {
+    /// Creates a new `Rs256` key verifier.
     pub fn with_rs256(key: K) -> Self {
         Self {
             key,
@@ -137,7 +128,7 @@ where
 
 #[cfg(feature = "sha2")]
 #[cfg_attr(docsrs, doc(cfg(feature = "sha2")))]
-impl<K> super::Verifier for RsaPublicKeyVerifier<K, crate::algorithm::Rs256>
+impl<K> super::Verifier for PublicKeyVerifier<K, crate::algorithm::Rs256>
 where
     K: PublicKey,
 {
